local M = {}

local helper = require('scenario/scenariohelper')
local reset = true
local start = false
local timer = 0
local continue = 0
local position = 0
local reversal = false

local function onScenarioChange(scenario)
	if scenario.state == 'pre-start' then
		reset = true
		start = false
		timer = 0
		continue = 0
		position = 0
		reversal = false
	end
end

local function success(speed)
    local result = {msg = 'You can\'t stop us - it is only a matter of time...'}
    scenarios.finish(result)
end

local function onRaceStart()
	start = true
	continue = 1
end

local function onBeamNGTrigger(data)
	if data.triggerName == 'portalEscape' then
		success(speed)
	end
end

local function onPreRender(dt)
	if start then
		timer = timer+dt
	end
	
	if position >= 300 then
		reversal = true
	elseif position <= -300 then
		reversal = false
	end
	
	if reversal then
		position = position-10
	else
		position = position+10
	end

	TorqueScript.eval([[
		lightning1.position = "]]..position..[[ 60.1608 104.725";
		lightning2.position = "]]..position..[[ 60.1608 102.725";
		lightning3.position = "]]..position..[[ 60.1608 106.725";
		lightning4.position = "]]..position..[[ 58.1608 104.725";
		lightning5.position = "]]..position..[[ 62.1608 104.725";
	]])
	
	if reset then
		TorqueScript.eval([[
			fire1.position = "10000 0 0";
			fire2.position = "10000 0 0";
			fire3.position = "10000 0 0";
			fire4.position = "10000 0 0";
			fire5.position = "10000 0 0";
			fire6.position = "10000 0 0";
			leak.position =  "10000 0 0";
			building11.position = "-17.3 149.995 -1000";
		]])
		be:reloadStaticCollision();
		reset = false
	end
	if continue == 1 and timer < 5 then
		ui_message('This facility was built in 1970, funded by the parent company, BTSF', 5, 'A')
		continue = 2
	elseif continue == 2 and timer > 5 then
		ui_message('Antimatter is created using a particle accelerator, which smashes atoms of hydrogen together at near light speed', 10, 'B')
		continue = 3
	elseif continue == 3 and timer > 15 then
		ui_message('Antimatter is highly reactive, and one atom has the explosive force of a small nuclear bomb', 8, 'C')
		continue = 4
	elseif continue == 4 and timer > 23 then
		ui_message('This container stores around 70,000 antimatter atoms', 6, 'D')
		continue = 5
	elseif continue == 5 and timer > 29 then
		ui_message('WARNING: VACUUM CORE HEAT INCREASING', 6, 'E')
		continue = 6
	elseif continue == 6 and timer > 30 then
		ui_message('Wait...what?!', 2, 'F')
		continue = 7
	elseif continue == 7 and timer > 32 then
		ui_message('No no no no no no no!', 2, 'G')
		continue = 8
	elseif continue == 7 and timer > 34 then
		ui_message('****!', 2, 'H')
		continue = 8
	elseif continue == 8 and timer > 35 then
		ui_message('WARNING: VACUUM CORE IMINENT - CORE UNSTABLE', 6, 'I')
		continue = 9
	elseif continue == 9 and timer > 36 then
		ui_message('This is not good!', 4, 'I')
		continue = 10
	elseif continue == 10 and timer > 40 then
		ui_message('I can\'t reset the system - it won\'t take input!', 4, 'J')
		continue = 11
	elseif continue == 11 and timer > 44 then
		ui_message('SEVERE FAILURE: CORE MELTED - VACUUM FIELD OFFLINE', 6, 'H')
		continue = 12
	elseif continue == 12 and timer > 45 then
		ui_message('The failsafe has been destroyed - someone planned this!', 5, 'I')
		continue = 13
	elseif timer >= 50 and continue == 13 then
		TorqueScript.eval([[
			fire1.position = "73.9581 73.9581 150.127";
			fire2.position = "-4.69762 31.1405 151.667";
			fire3.position = "53.686 96.4132 151.922";
			fire4.position = "30.4169 11.6162 22.5564";
			fire5.position = "8.92428 101.452 23.1604";
			fire6.position = "76.0529 36.656 25.7894;
		]])
		continue = 14
	elseif timer > 50 then
		TorqueScript.eval([[
			building11.position = "-17.3 149.995 65.54";
		]])
		be:reloadStaticCollision();
		ui_message('Get out of here! DRIVE!!! - Get to the green portal! Now!', 600, 'I')
		continue = 15
	end
	if timer >= 60 and continue == 15 then
		TorqueScript.eval([[
			leak.position =  "29.4354 59.7585 80.6312";
		]])
	end
end

M.onBeamNGTrigger = onBeamNGTrigger
M.onScenarioChange = onScenarioChange
M.onPreRender = onPreRender
M.onRaceStart = onRaceStart
return M