local M = {}

local helper = require('scenario/scenariohelper')
local reset = true
local start = false
local magnets = 0
local continue = 1
local timer = 0
local position = 0
local reversal = false
local blackhole = {["expandHorizon"] = true, ["expandAccDisc"] = true, ["removeObjects"] = true, ["shutdown"] = false, ["phase2"] = false, ["scale"] = 0, ["rotation"] = 0, ["shrink"] = 19}

local function onScenarioChange(scenario)
	if scenario.state == 'pre-start' then
		reset = true
		start = false
		magnets = 0
		continue = 1
		reversal = false
		timer = 0
		position = 0
		blackhole = {["expandHorizon"] = true, ["expandAccDisc"] = true, ["removeObjects"] = true, ["shutdown"] = false, ["phase2"] = false, ["scale"] = 0, ["rotation"] = 0, ["shrink"] = 19}
		local player = scenetree.findObject('scenario_player0')
		player:queueLuaCommand('obj:setWind(0, 0, 0)')
	end
end

local function success(speed)
    local result = {msg = '...'}
    scenarios.finish(result)
end

local function onRaceStart()
	start = true
end

local function onBeamNGTrigger(data)
	if data.triggerName == 'active1' and data.subjectName == 'key_box_1' then
		ui_message('MAGNET A NOW ONLINE', 5, 'A')
		magnets = magnets + 1
		TorqueScript.eval([[
			key_box_1.position = "10000 0 0";
		]])
	elseif data.triggerName == 'active2' and data.subjectName == 'key_box_2' then
		ui_message('MAGNET B NOW ONLINE', 5, 'A')
		magnets = magnets + 1
		TorqueScript.eval([[
			key_box_2.position = "10000 0 0";
		]])
	elseif data.triggerName == 'stage2' and blackhole["phase2"] then
		blackhole["shutdown"] = true
		TorqueScript.eval([[
			objectMarker.position = "10000 0 0";
		]])
	elseif data.triggerName == 'death' and blackhole["scale"] >= 10 then
		success(speed)
	end
end

local function updateBlackhole()
	if blackhole["removeObjects"] then
		TorqueScript.eval([[
			fire1.position = "10000 0 0";
			fire2.position = "10000 0 0";
			fire3.position = "10000 0 0";
			fire4.position = "10000 0 0";
			fire5.position = "10000 0 0";
			fire6.position = "10000 0 0";
			fire7.position = "10000 0 0";
			fire8.position = "10000 0 0";
		]])
		blackhole["removeObjects"] = false
	end
	if blackhole["expandAccDisc"] then
		blackhole["scale"] = blackhole["scale"]+0.01
		TorqueScript.eval([[
			accretionDisc.scale = "]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[";
		]])
		if blackhole["scale"] >= 5 then
			blackhole["expandAccDisc"] = false
			be:reloadStaticCollision();
		end
	end
	if blackhole["expandHorizon"] then
		TorqueScript.eval([[
			eventHorizon.scale = "]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[";
		]])
		if blackhole["scale"] >= 5 then
			blackhole["expandHorizon"] = false
		end
	end
	
	blackhole["rotation"] = blackhole["rotation"]+1
	TorqueScript.eval([[
		accretionDisc.rotation = "0 0 1 ]]..blackhole["rotation"]..[[";
	]])
	
	if blackhole["scale"] >= 1 and blackhole["shrink"] > 0 then
		blackhole["shrink"] = blackhole["shrink"]-0.05
		TorqueScript.eval([[
			building1.scale = "]]..((2/19)*blackhole["shrink"])..[[ ]]..((7/19)*blackhole["shrink"])..[[ ]]..((10/19)*blackhole["shrink"])..[[";
			building7.scale = "]]..((2/19)*blackhole["shrink"])..[[ ]]..((7/19)*blackhole["shrink"])..[[ ]]..((10/19)*blackhole["shrink"])..[[";
			building9.scale = "]]..((2/19)*blackhole["shrink"])..[[ ]]..((7/19)*blackhole["shrink"])..[[ ]]..((10/19)*blackhole["shrink"])..[[";
			building2.scale = "]]..((2/19)*blackhole["shrink"])..[[ ]]..((5/19)*blackhole["shrink"])..[[ ]]..((0.7/19)*blackhole["shrink"])..[[";
			building4.scale = "]]..((2/19)*blackhole["shrink"])..[[ ]]..((5/19)*blackhole["shrink"])..[[ ]]..((0.7/19)*blackhole["shrink"])..[[";
			building5.scale = "]]..((2/19)*blackhole["shrink"])..[[ ]]..((5/19)*blackhole["shrink"])..[[ ]]..((0.7/19)*blackhole["shrink"])..[[";
			building6.scale = "]]..((2/19)*blackhole["shrink"])..[[ ]]..((5/19)*blackhole["shrink"])..[[ ]]..((0.7/19)*blackhole["shrink"])..[[";
			strut1.scale =    "]]..((4/19)*blackhole["shrink"])..[[ ]]..((4/19)*blackhole["shrink"])..[[ ]]..((1/19)*blackhole["shrink"])..[[";	
			strut2.scale =    "]]..((4/19)*blackhole["shrink"])..[[ ]]..((4/19)*blackhole["shrink"])..[[ ]]..((1/19)*blackhole["shrink"])..[[";
			strut3.scale =    "]]..((4/19)*blackhole["shrink"])..[[ ]]..((4/19)*blackhole["shrink"])..[[ ]]..((1/19)*blackhole["shrink"])..[[";	
			strut4.scale =    "]]..((4/19)*blackhole["shrink"])..[[ ]]..((4/19)*blackhole["shrink"])..[[ ]]..((1/19)*blackhole["shrink"])..[[";
			strut5.scale =    "]]..((4/19)*blackhole["shrink"])..[[ ]]..((4/19)*blackhole["shrink"])..[[ ]]..((1/19)*blackhole["shrink"])..[[";	
			strut6.scale =    "]]..((4/19)*blackhole["shrink"])..[[ ]]..((4/19)*blackhole["shrink"])..[[ ]]..((1/19)*blackhole["shrink"])..[[";
			strut7.scale =    "]]..((4/19)*blackhole["shrink"])..[[ ]]..((4/19)*blackhole["shrink"])..[[ ]]..((1/19)*blackhole["shrink"])..[[";	
			strut8.scale =    "]]..((4/19)*blackhole["shrink"])..[[ ]]..((4/19)*blackhole["shrink"])..[[ ]]..((1/19)*blackhole["shrink"])..[[";	
			strut9.scale =    "]]..((4/19)*blackhole["shrink"])..[[ ]]..((4/19)*blackhole["shrink"])..[[ ]]..((1/19)*blackhole["shrink"])..[[";
			strut10.scale =   "]]..((4/19)*blackhole["shrink"])..[[ ]]..((4/19)*blackhole["shrink"])..[[ ]]..((1/19)*blackhole["shrink"])..[[";
			strut11.scale =   "]]..((4/19)*blackhole["shrink"])..[[ ]]..((4/19)*blackhole["shrink"])..[[ ]]..((1/19)*blackhole["shrink"])..[[";		
			strut12.scale =   "]]..((4/19)*blackhole["shrink"])..[[ ]]..((4/19)*blackhole["shrink"])..[[ ]]..((1/19)*blackhole["shrink"])..[[";
			stabiliser1.scale = "]]..((3/19)*blackhole["shrink"])..[[ ]]..((3/19)*blackhole["shrink"])..[[ ]]..((3/19)*blackhole["shrink"])..[[";
			stabiliser2.scale = "]]..((3/19)*blackhole["shrink"])..[[ ]]..((3/19)*blackhole["shrink"])..[[ ]]..((3/19)*blackhole["shrink"])..[[";
			core.scale =        "]]..((3/19)*blackhole["shrink"])..[[ ]]..((3/19)*blackhole["shrink"])..[[ ]]..((15/19)*blackhole["shrink"])..[[";
		]])
		if blackhole["shrink"] == 0 then
			be:reloadStaticCollision();
		end
	end
	
	if blackhole["shutdown"] and not blackhole["energise"] then
		if position >= 300 then
			reversal = true
		elseif position <= -300 then
			reversal = false
		end
	
		if reversal then
			position = position-20
		else
			position = position+20
		end

		TorqueScript.eval([[
			lightning1.position = "]]..position..[[ 734.031 105.928";
			lightning2.position = "]]..position..[[ 734.031 100.725";
			lightning3.position = "]]..position..[[ 734.031 110.725";
			lightning4.position = "]]..position..[[ 729.1608 105.928";
			lightning5.position = "]]..position..[[ 739.1608 105.928";
		]])
		
		blackhole["scale"] = blackhole["scale"]-0.01
		TorqueScript.eval([[
			accretionDisc.scale = "]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[";
			eventHorizon.scale =  "]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[";
		]])
		
		if blackhole["scale"] <= 0 then
			TorqueScript.eval([[
				ambient1.position = "10000 0 0";
			]])
			blackhole["scale"] = 0
			blackhole["energise"] = true
		end
	end
end

local function onPreRender(dt)
	if start then
		updateBlackhole()
	end
	
	if magnets == 4 then
		ui_message('Get a safe distance away; I will increase the magnetic field strength of the magnets', 10, 'B')
		TorqueScript.eval([[
			objectMarker.position = "30.3556 175.006 -3.98867";
		]])
		blackhole["phase2"] = true
		magnets = 5
	end
	
	if blackhole["energise"] then
		timer = timer+dt
		TorqueScript.eval([[
			lightning1.position = "32.8678 741.731 80.5791";
			lightning2.position = "32.8678 741.731 80.5791";
			lightning3.position = "32.8678 741.731 80.5791";
			lightning4.position = "32.8678 741.731 80.5791";
			lightning5.position = "32.8678 741.731 80.5791";
		]])
		if timer >= 1 and continue == 1 then
			ui_message('', 3, 'A')
			ui_message('Something\'s not right...It\'s become too powerful - reverse and drive in the opposite direction...and fast!!!', 10, 'C')
			continue = 2
		elseif timer >= 10 then
			if blackhole["scale"] <= 10 then
				blackhole["scale"] = blackhole["scale"]+0.1
			end
			
			TorqueScript.eval([[
				accretionDiscEBack.position = "32.8678 742 80.5791";
				accretionDiscE.position = "32.8678 741.731 80.5791";
				accretionDiscEBack.scale = "]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[";
				accretionDiscE.scale = "]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[ ]]..blackhole["scale"]..[[";
				accretionDiscE.rotation = "0 -1 0 ]]..blackhole["rotation"]..[[";
			]])
			local player = scenetree.findObject('scenario_player0')
			player:queueLuaCommand('obj:setWind(0, 200, 50)')
		end
	end
	
	if reset then
		TorqueScript.eval([[
			scenario_player0.position = "-12.486 -39.9743 0.184686";
			scenario_player0.rotation = "0 0 1 180";
			key_box_1.position = "-372.398 505.801 0.581412";
			key_box_1.rotation = "0 0 1 180";
			key_box_2.position = "470.453 515.521 -1.1254e-005";
			key_box_2.rotation = "0 0 1 180";
			fire1.position = "2.06605 723.841 102.307";
			fire2.position = "49.5307 720.722 47.9319";
			fire3.position = "44.0665 758.344 33.5691";
			fire4.position = "-15.2996 741.942 34.5491";
			fire5.position = "-4.62193 701.501 144.868";
			fire6.position = "18.677 746.923 161.748";
			fire7.position = "17.7104 795.703 146.576";
			fire8.position = "33.2451 684.306 138.164";
			lightning1.position = "10000 0 0";
			lightning2.position = "10000 0 0";
			lightning3.position = "10000 0 0";
			lightning4.position = "10000 0 0";
			lightning5.position = "10000 0 0";
			accretionDisc.scale = "1 1 1";
			eventHorizon.scale = "1 1 1";
			objectMarker.position = "10000 0 0";
			building1.scale = "2 7 10";
			building2.scale = "2 5 0.7";
			building3.scale = "2 7 10";
			building4.scale = "2 5 0.7";
			building5.scale = "2 5 0.7";
			building6.scale = "2 5 0.7";
			building7.scale = "2 7 10";
			building8.scale = "2 7 10";
			building9.scale = "2 7 10";
			strut1.scale = "4 4 1";
			strut2.scale = "4 4 1";
			strut3.scale = "4 4 1";
			strut4.scale = "4 4 1";
			strut5.scale = "4 4 1";
			strut6.scale = "4 4 1";
			strut7.scale = "4 4 1";
			strut8.scale = "4 4 1";
			strut9.scale = "4 4 1";
			strut10.scale = "4 4 1";
			strut11.scale = "4 4 1";
			strut12.scale = "4 4 1";
			stabiliser1.scale = "3 3 3";
			stabiliser2.scale = "3 3 3";
			core.scale = "3 3 15";
			lightning1.position = "10000 0 0";
			lightning2.position = "10000 0 0";
			lightning3.position = "10000 0 0";
			lightning4.position = "10000 0 0";
			lightning5.position = "10000 0 0";
			accretionDiscE = "10000 0 0";
			accretionDiscE.scale = "1 1 1";
			accretionDiscEBack = "10000 0 0";
			accretionDiscEBack.scale = "1 1 1";
			ambient1.position = "21.2767 406.61 69.2829";
		]])
		be:reloadStaticCollision();
		reset = false
	end
end

M.onBeamNGTrigger = onBeamNGTrigger
M.onScenarioChange = onScenarioChange
M.onPreRender = onPreRender
M.onRaceStart = onRaceStart
return M