local M = {}

local helper = require('scenario/scenariohelper')
local rotation = {["d_I"] = 0, ["d_J"] = 0, ["d_K"] = 0, ["b_I"] = 0, ["b_J"] = 0}
local active = {["d2"] = false, ["d_a1"] = false}
local reset = true
local cube = {['key_box_1'] = false, ['key_box_2'] = false}

local function onScenarioChange(scenario)
	if scenario.state == 'pre-start' then
		file = io.open("log_cube1B.txt", "w")
		file:close()
		reset = true
		active = {["d2"] = false, ["d_a1"] = false}
		rotation = {["d_I"] = 0, ["d_J"] = 0, ["d_K"] = 0, ["b_I"] = 0, ["b_J"] = 0}
		cube = {['key_box_1'] = false, ['key_box_2'] = false}
	end
end

local function onRaceWaypointReached(data)
	file = io.open("log_cube1B.txt", "w")
	file:write("Waypoint Activated: "..data.waypointName.."\n")
	file:write("Vehicle Used: "..data.vehicleName)
	file:close()
	if data.vehicleName == 'scenario_player0' then
		if data.waypointName == 'Waypoint1' then
			helper.flashUiMessage('This wire is not connected...', 5) 
		elseif data.waypointName == 'Waypoint2' then
			helper.flashUiMessage('This is an advanced divider; you need 2 power sources', 5)  
		end
	end
end


local function onBeamNGTrigger(data)
	if data.triggerName == 'circuitBreaker1' then
		if not active["d2"] then
			TorqueScript.eval([[
				circuit1Unpowered.position = "10000 0 0";
				circuit1Powered.position = "145.945 23.3926 39.086";
				door1.position = "10000 0 0";
			]])
			be:reloadStaticCollision();
			active["d2"] = true
		else
			TorqueScript.eval([[
				circuit1Powered.position = "10000 0 0";
				circuit1Unpowered.position = "145.945 23.3926 39.086";
				door1.position = "370.077 118.436 -0.184723";
			]])
			be:reloadStaticCollision();
			active["d2"] = false
		end
	elseif data.triggerName == 'circuitBreaker2' then
		if cube['key_box_2'] then
			cube['key_box_2'] = false
		end
		if not active["d_a1"] then
			TorqueScript.eval([[
				circuit2Unpowered.position = "10000 0 0";
				circuit2Powered.position = "366.061 62.6028 65.0794";
			]])
			active["d_a1"] = true
		else
			TorqueScript.eval([[
				circuit2Powered.position = "10000 0 0";
				circuit2Unpowered.position = "366.061 62.6028 65.0794";
			]])
			active["d_a1"] = false
		end
	elseif data.triggerName == 'cube1' and data.subjectName == 'scenario_player0' then
		cube['key_box_1'] = true
	end
end

local function transportCube(cube)
	player = scenetree.findObject('scenario_player0')
	
	if cube == 'key_box_1' then
		TorqueScript.eval([[
			key_box_1.position = "]]..player:getPosition().x..[[ ]]..player:getPosition().y..[[ ]]..(player:getPosition().z+2)..[[";
			key_box_1.rotation = "1 0 0 0";
		]])
	end
end

local function rotateObjects()
	--Update rotations
	--Divider
	rotation["d_I"] = rotation["d_I"]+4
	rotation["d_J"] = rotation["d_J"]+6
	rotation["d_K"] = rotation["d_K"]+8
	--Battery - all should be on constantly
	rotation["b_I"] = rotation["b_I"]+5
	rotation["b_J"] = rotation["b_J"]+10
	
	TorqueScript.eval([[
		batteryDisc1.rotation = "0 ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[";
		batteryDisc2.rotation = "0 ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[";
		batteryDisc3.rotation = "0 ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[";
		batteryDisc4.rotation = "0 ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[";
		batteryDisc5.rotation = "0 ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[";
		batteryDisc6.rotation = "0 ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[";

		dividerDisc1_1.rotation = "0 ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[";
		dividerDisc1_2.rotation = "0 ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[";
	]])
	
	if active["d2"] then
		TorqueScript.eval([[
			dividerDisc2_1.rotation = "0 ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[";
			dividerDisc2_2.rotation = "0 ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[";
		]])
	end
	if active["d_a1"] then
		TorqueScript.eval([[
			dividerDiscA1.rotation = "0 ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[";
			dividerDiscA2.rotation = "0 ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[";
			dividerDiscA3.rotation = "0 ]]..rotation["d_K"]..[[ ]]..rotation["d_K"]..[[ ]]..rotation["d_K"]..[[";
		]])
	end
end

local function onPreRender(dt)
	if reset then
		TorqueScript.eval([[
			door1.position = "181.139 116.034 -0.184723";
		]])
		reset = false
	end

	rotateObjects()
	
	if cube['key_box_1'] then
		transportCube('key_box_1')
	end
end

M.onRaceWaypointReached = onRaceWaypointReached
M.onScenarioChange = onScenarioChange
M.onPreRender = onPreRender
M.onBeamNGTrigger = onBeamNGTrigger
return M